//============================================================================
// FILE: C_ActivitySet.h
//
// TYPE: C++CLASS
// DESCRIPTION: entête des ensembles d'ectivités (classes 'moteur')
// 
// AUTHORS: Rellier (INRA/UBIA, Toulouse)
// REFERENCE : "Fondements ontologiques des systèmes pilotés"
//             R. Martin-Clouaire, J.-P. Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _C_ACTIVITYSET_HH
#define _C_ACTIVITYSET_HH

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//
#include "C_CoreTypes.h"
#include "C_ResourceAllocation.h"

// EXTERNAL REFERENCES

/**@name Ensemble d'activités (non activité lui-même)
\begin{itemize} \item Directive d'inclusion : {\tt \#include C\_ActivitySet.h} \end{itemize}
*/ 
//@{

/**@name Ensemble simple d'activités (primitive ou non)*/ 
//@{
        ///
class ActivitySet : public Entity // _no swig
{
public:
          // Constructeur de base. 
  ActivitySet();   
          // Constructeur par copie.
  ActivitySet(const ActivitySet&);

protected :    
};

//@}  // fin ActivitySet

//===========================================================================
//
// INSTRUCTION_SET
//
//===========================================================================

/**@name Ensemble (jeu) d'instructions : classe mère */ 
//@{
        ///
class InstructionSet : public Entity // _no swig_
{
public:
          // Constructeur de base. 
  InstructionSet();
          // Constructeur par copie.
  InstructionSet(InstructionSet&);
          // Destructeur.
  ~InstructionSet();

  void DeleteInDepth();  // dans C_ActivityNonPrimitive

           // VRAI si 2 Single ou 2 Multiple ; FAUX sinon.
  Boolean SameFamily(InstructionSet*);
           // VRAI si Single ; FAUX sinon.
  Boolean IsSingle();
           //
  Boolean HasSingleRequiredAsElement();  
  // + JPR 061208 (because suppr. classe SingleRequiredIS)
  // remplace HasAsElement(SINGLE_REQUIRED_INSTRUCTION_SET)
           // JPR 100315
  Boolean HasSingleWithOperationToContinueAsElement();
           //
  void SetType(int);
           //
  int GetType();

           // 
  void  SpecialDisplay();

           // 
  void  SpecialDisplay(int k);

    // comptage Const/Dest
    static int   getNbConst() {return _nbConst;}
    static int   getNbDest() {return _nbDest;}     

protected :    

   int static   _nbConst;
   int static   _nbDest;
 };

//@}  // fin InstructionSet

//===========================================================================
//
// SINGLE_INSTRUCTION_SET
//
//===========================================================================

/**@name Ensemble (jeu) d'une seule instruction : classe mère
*/ 
//@{
        ///
class SingleInstructionSet : public InstructionSet // _no swig_
{
public:
          // Constructeur de base. 
  SingleInstructionSet();
          // Constructeur par copie.
  SingleInstructionSet(SingleInstructionSet&);  // pas const because getComponent used 100909
  //SingleInstructionSet(const SingleInstructionSet&);

          // Destructeur, avec destruction de l'allocation.   // + JPR 100909
  void DeleteInDepth();
  void DeleteInDepth100917();
  void DeleteInDepth100918();

          /// Opérateur de test d'égalité. Les deux ensembles sont égaux (et l'opérateur renvoie 1) si pour tous ses éléments sont 'égaux', au sens de leur opérateir '=='. Sinon l'opérateur renvoie 0.
  int  operator == (SingleInstructionSet& source);  // gcc 4.1

           //
  void               SetAct(PrimitiveActivity*);
           //
  PrimitiveActivity* GetAct();
           //
  Operation*         GetAllocatedOperation();
           //
  InstructionSet*    ReducedCopy(); // + JPR 030805

  /**@name Méthodes d'allocation de ressources
Ces fonctions établissent une liste d'allocations valides pour l'unique (par définition et construction) activité primitive (dite {\em act}) du présent jeu. Ces allocations sont compatibles avec les allocations établies pour d'éventuelles autres activités figurant avant {\em act} dans la conjonction de jeux {\em set} dont le présent jeu est un élément.\\
Quelle que soit la méthode, la liste renvoyée est vide si une allocation viole une contrainte ou bien si la spécification requiert au moins une allocation non vide (cas des fonctions ANY et SETS avec un effectif non nul) et qu'on n'a pas trouvé d'allocation non vide. La liste contient une seule allocation vide si l'activité ne comporte pas de spécification d'opérateur. Elle peut contenir une seule allocation vide si la spécification ne requiert pas au moins une allocation non vide (cas des fonctions ANY et SETS avec un effectif nul et des fonctions MAX et DISJ).
*/ 
//@{
           /// Ici, les allocations portent sur les objets opérés, l'opération, les ressources propres de l'opération et l'opérateur.\index{méthodes!GetSpecimens}
  pEntityTab*  GetSpecimens(InstructionSetConjunction* pSet,
								   InstructionSetAllocation* pU_alloc);
  pEntityTab*  GetSpecimensForTransfer(InstructionSetConjunction* pSet,
								   InstructionSetAllocation* pU_alloc);

           /// Ici, les allocations ne portent que sur l'opération en jeu, les ressources propres de l'opération et sur l'opérateur.\index{méthodes!GetOtRscPerfSpecimens} 
  pEntityTab*  GetOtRscPerfSpecimens(InstructionSetConjunction* pSet,
								   InstructionSetAllocation* pU_alloc);

           /// Ici, les allocations ne portent que sur les ressources propres de l'opération et sur l'opérateur.\index{méthodes!GetRscPerfSpecimens} 
  pEntityTab*  GetRscPerfSpecimens(InstructionSetConjunction* pSet,
								   InstructionSetAllocation* pU_alloc);

           /// Ici, les allocations ne portent que sur l'opérateur.\index{méthodes!GetPerfSpecimens} 
  pEntityTab*  GetPerfSpecimens(InstructionSetConjunction* pSet,
								InstructionSetAllocation* pU_alloc);

//@}  // fin Get..Specimens

protected :
 };

//@}  // fin SingleInstructionSet

// ===========================================================================

/**@name Ensemble d'une seule instruction primitive */ 
//@{
        ///
class SinglePrimitiveInstructionSet : public SingleInstructionSet // _no swig_
{
public:
          // Constructeur de base. Donne la valeur C\_PRIM au type.
  SinglePrimitiveInstructionSet();
          // Constructeur par copie.
  SinglePrimitiveInstructionSet(SinglePrimitiveInstructionSet&);

private :
 };

//@}  // fin SinglePrimitiveInstructionSet

//===========================================================================
//
// MULTIPLE_INSTRUCTION_SET
//
//===========================================================================

/**@name Ensemble de jeux d'instructions : classe mère */ 
//@{
        ///
class MultipleInstructionSet : public InstructionSet // _no swig_
{
public:
          // Constructeur de base. 
  MultipleInstructionSet();
          // Constructeur par copie.
  MultipleInstructionSet(MultipleInstructionSet&);

          /// Opérateur de test d'égalité. Les deux ensembles sont égaux (et l'opérateur renvoie 1) si pour tous ses éléments sont 'égaux', au sens de leur opérateir '=='. Sinon l'opérateur renvoie 0.
  int  operator == (const MultipleInstructionSet& source); // gcc 4.1

protected :
 };

//@}  // fin MultipleInstructionSet

// ===========================================================================

/**@name Disjonction d'instructions ou de jeux d'instructions */ 
//@{
        ///
class InstructionSetDisjunction : public MultipleInstructionSet // _no swig_
{
public:
          // Constructeur de base. Donne la valeur C\_DISJ au type.
  InstructionSetDisjunction();
          // Constructeur par copie.
  InstructionSetDisjunction(InstructionSetDisjunction&);

           //
  InstructionSet*  ReducedCopy();

           // Si la liste des éléments de this est (a b) et que celle de arg est (c d), alors la liste des éléments de this devient (a b c d). 
  void SumEqual(InstructionSet* arg);

           // Si la liste des éléments de this est (a b) et que celle de arg est (c d), alors la liste des éléments de this devient (a b c' d'), où c' et d' sont des copies de c et d. La désallocation de la mémoire allouée à l'argument {\em arg} est sous la responsabilité du code appelant.
  void SumCopyEqual(InstructionSet* arg);

           // Si la liste des éléments de this est (a b) et que celle de arg est (c d), alors la liste des éléments de this devient ((a c) (a d) (b c) (b d)).
  void ProdEqual(InstructionSet* arg);

           // Si la liste des éléments de this est (a b) et que celle de arg est (c d), alors la liste des éléments de this devient ((a c) (a d) (b c) (b d)). La désallocation de la mémoire allouée à l'argument {\em arg} est sous la responsabilité du code appelant.
  void ProdCopyEqual(InstructionSet* arg);

           // 
  void  ExtendToSeveralAllocations(); // ex InstructionSet::PreAllocate

           // Remplace pElement par les éléments de substitute à partir du même indice. Version de replaceElement adaptée à des InstructionSet.
  Boolean ReplaceSubSet(BasicEntity* pElement, 
						pEntityTab substitute, 
						pEntityTab compList);
 
           // 
  InstructionSetDisjunction*  GetConsistentUpperSubsets();
           //
  Boolean  ExistsSetIncludingArgumentSet(InstructionSetConjunction*, Boolean);
           //
  Boolean  ExistsMAXSpecIncludingArgumentMAXSpec(InstructionSetConjunction*, Boolean);
           //
  Boolean  ExistsSetIncludingArgumentMAXSpec(InstructionSetConjunction*, Boolean);

  int a;
 };

//@}  // fin InstructionSetDisjunction

// ===========================================================================

/**@name Conjonction d'instructions ou de de jeux d'instructions */ 
//@{
        ///
class InstructionSetConjunction : public MultipleInstructionSet // _no swig_
{
public:
          // Constructeur de base. Donne la valeur C\_CONJ au type.
  InstructionSetConjunction();
          // Constructeur par copie.
  InstructionSetConjunction(InstructionSetConjunction&);
  
           // obsolete : to scratch
  pEntityTab* ScanForInstAllocations();
           // appel imprévu non géré
  InstructionSet*  ReducedCopy();
           //
  pEntityTab*  ReturnNonInconsistentSubSets(pEntityTab*);
           //
  pEntityTab*  ReturnReducedOnMAXSpecSubSets(pEntityTab*, pEntityTab&, Entity*);
           //
  pEntityTab*  ReturnReducedOnMAXSpecCloneSets(Entity*, Entity*, 
											   pEntityTab*, int, 
											   Entity*, UserClassId);
           //
  //void  RemoveLattice();
           //
  void  ShallowDisplay();
           //
  Boolean  HasSuperNodes();
           //
  InstructionSetConjunction*  GetConsistentUpperSubsets();
           /// Cherche au premier niveau puis en profondeur dans les éléments de cette conjonction, et renvoie, le jeu d'une unique activité dont l'activité est celle passée en paramètre. \index{méthodes!GetSingle}
  SingleInstructionSet* GetSingle(PrimitiveActivity*);
           ///
  void AugmentInstructionSetAllocation(InstructionSetAllocation*);
           ///
  void AllocateInstruction(InstructionAllocation*, 
			   PrimitiveActivity*);

           // Renvoie la sous-liste constituée des premiers éléments de la présente conjonction, jusqu'à et y compris celle passée en paramètre.
  pEntityTab* GetLeadingSingles(SingleInstructionSet*);

           // Renvoie la sous-liste constituée des premiers éléments de la présente conjonction, jusqu'à et y compris celle dont le rang est passé en paramètre.
  pEntityTab* GetLeadingSingles(int);

           /// Renvoie la liste de tous les critères correspondant à un but de la classe spécifiée par l'argument (e.g. TARGET_GOAL), si ce but est poursuivi par au moins une activité du jeu. Les critères sont instanciées par la présente fonction.
  pEntityTab* CriteriaRestriction(UserClassId);

  
  /**@name Methodes prédéfinies d'ordonnancement des activités */
  //@{
           /// Pour chaque activité sous-jacente, si elle possède au moins une de ses spécifications dotée de l'opérateur MAX, elle est repoussée en fin de liste. \index{méthodes!PushMaxSpecToBack (InstructionSetConjunction)}
  void     PushMaxSpecToBack();

           /// Trie les activités par force de priorité d'allocation décroissante. \index{méthodes!SortByAllocPriority (InstructionSetConjunction)}
  void     SortByAllocPriority();

           /// Trie les activités par force de priorité d'allocation décroissante, puis pour chaque activité sous-jacente, si elle possède au moins une de ses spécifications dotée de l'opérateur MAX, elle est repoussée en fin de liste. \index{méthodes!PushMaxSpecToBackThenPushMaxSpecToBack (InstructionSetConjunction)}
  void     SortByAllocPriorityThenPushMaxSpecToBack();

           /// Pour chaque activité sous-jacente, si elle possède au moins une de ses spécifications dotée d'un opérateur différent de NEW\_ENTITIES (i.e SAME\_PREVIOUS\_ENTITIES, SAME\_FIRST\_ENTITIES ou EXACTLY\_SAME) pour une de ses spécifications, elle est remontée en début de liste. \index{méthodes!PullSameEntitySpecToFront (InstructionSetConjunction)}
  void     PullSameEntitySpecToFront();

           // Pour chaque activité sous-jacente, si elle possède au moins une de ses spécifications dotée de l'opérateur MAX, elle est remontée en tête de liste.
  void     PullMaxSpecToFront();

           // Trie les activités par force de priorité d'allocation décroissante, puis pour chaque activité sous-jacente, si elle possède au moins une de ses spécifications dotée de l'opérateur MAX, elle est remontée en tête de liste.
  void     SortByAllocPriorityThenPullMaxSpecToFront();
  //@}

           // Chaque activité sous-jacente qui possède au moins une de ses spécifications dotée de l'opérateur MAX est ajoutée à la liste retournée.
  pEntityTab*     GetMAXSpecifiedActivityList();   // still unused

           // 
  PrimitiveActivity*   SelectMAXSpecifiedPrimitiveWithHigherPriority();

           // 
  SingleInstructionSet*   SelectSingleOfMAXSpecifiedPrimitiveWithHigherPriority();
           // 
  Entity*   SelectSpecOfMAXSpecifiedPrimitiveWithHigherPriority();
};
 	
  int stronger_priority_first(BasicEntity*, BasicEntity*);

//@}  // fin InstructionSetConjunction

// ===========================================================================

/**@name Conjonction décomposable d'instructions ou de jeux d'instructions */ 
//@{
      ///
class SplittableInstructionSet : public InstructionSetConjunction // _no swig_
{
public:
          // Constructeur de base. Donne la valeur C\_SPLIT au type.
  SplittableInstructionSet();
          // Constructeur par copie.
  SplittableInstructionSet(SplittableInstructionSet&);

           //
  InstructionSet*  ReducedCopy();

           //
  pEntityTab*  Split();   // tableau de InstructionSet

private :
 };

//@}  // fin SplittableInstructionSet

// ===========================================================================

/**@name Conjonction non décomposable d'instructions ou de jeux d'instructions */ 
//@{
        ///
class UnsplittableInstructionSet : public InstructionSetConjunction // _no swig_
{
public:
          // Constructeur de base. Donne la valeur C\_UNSPLIT au type.
  UnsplittableInstructionSet();
          // Constructeur par copie.
  UnsplittableInstructionSet(UnsplittableInstructionSet&);

           //
  InstructionSet*  ReducedCopy();

           //
  pEntityTab*  Split();   // tableau de InstructionSet

private :
 };

//@}  // fin UnsplittableInstructionSet



//===========================================================================

//@}  // fin ensemble d'instructions






#endif  // _C_ACTIVITYSET_H
