//============================================================================
// FILE: C_CriterionSet.h
//
// TYPE: C++CLASS
// DESCRIPTION: entête des ensembles de critères (classes 'moteur')
// 
// AUTHORS: Rellier (INRA/UBIA, Toulouse)
// REFERENCE : "Fondements ontologiques des systèmes pilotés"
//             R. Martin-Clouaire, J.-P. Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _C_CRITERIONSET_HH
#define _C_CRITERIONSET_HH

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//
#include "C_CoreTypes.h"
#include "C_ResourceAllocation.h"

// EXTERNAL REFERENCES

/**@name Les critères considérés conjointement
\begin{itemize} \item Directive d'inclusion : {\tt \#include C\_CriterionSet.h} \end{itemize}
*/ 
//@{

/**@name Ensembles de jeux de critères
*/ 
//@{

/**@name La classe de base
*/ 
//@{
        ///
//class CriterionSet : public DescriptedEntity // _no swig_
/*
  modif JPR 090819 : en 5.0, WriteToCls (appelé dans mainMakeCls) plante
sur la récupération du tableau de Méthod, après casting de pObj en Entity*, 
alors que Criterionset est une DescriptedEntity.
Ce plantage n'est pas observé jusqu'à la 4.9, avec le même core de WriteToCls.

Or ce plantage semble normal. On fait donc en sorte que le casting ne pose
pas de pb : CriterionSet devient une Entity.
*/
class CriterionSet : public Entity // _no swig_
{
public:
          // Constructeur de base. 
  CriterionSet();
          // Constructeur par copie.
  CriterionSet(const CriterionSet&);
          // Destructeur.
  ~CriterionSet();

           // 
  void  DeleteInDepth();
           // 
  void  RemoveLattice();
           // 
  pEntityTab* GetAllCriterionSubsets();
           // 
  void  SpecialDisplay();
           // 
  void  SpecialDisplay(int k);

           /// Crée un nouveau treillis de critères dont le noeud racine est le {\bf CriterionSet} renvoyé. Si le {\bf CriterionSet} objet du message (this) est vide, le nouveau noeud racine ne contient que le critère en argument, et this est désalloué. Sinon, le nouveau noeud racine contient this parmi la liste de ses sous-noeuds : this n'est donc pas désalloué.\index{méthode!ExtendLatticeWithCriterion}
  CriterionSet*  ExtendLatticeWithCriterion(Criterion*, float);
           //
  CriterionSet*  FindSetInLattice(pEntityTab*);  // tableau de critères
           //
  void  DisplayLattice();
  void  DisplayLattice(int k);
           //
  CriterionSet*  GetDownRemoving(Criterion*); 
           // 
  CriterionSet*  ExtractNodeCopy(pEntityTab*); // tableau de critères

    // comptage Const/Dest
    static int   getNbConst() {return _nbConst;}
    static int   getNbDest() {return _nbDest;}     

protected :    

   int static   _nbConst;
   int static   _nbDest; 
 };

//@}  // fin classe CriterionSet

// ===========================================================================

/**@name Fonctions */
//@{
           ///
void MakeFinalWeightLattice();
           ///
void MakeWeightLattice();
           ///
void DeleteFinalWeightLattice();
           ///
void DeleteWeightLattice();
//@}


//@}  // fin CriterionSet


//@}  // fin criteres conjoints

#endif  // _C_CRITERIONSET_HH
