#!/bin/bash
MAJOR=5
MINOR=9

# check the local shell
#----------------------
myshell=sh
echo $SHELL | grep csh > tmpInstallDIESE
if [ -s tmpInstallDIESE ]
then
  myshell=csh
fi
rm -f tmpInstallDIESE
if [ $myshell = csh ]
then
  echo "||| "
  echo "||| Please run 'install_DIESE_lib.csh' instead."
  echo "||| "
  exit
fi

SOURCE=$PWD

cd "$HOME"
# customize .defEnvDIESE_csh
#-----------------------------------
onContinue=1
while([ $onContinue = 1 ]) do 
# ... {
if [ ! -e .defEnvDIESE_sh  ]
then
  /bin/cp "$SOURCE"/.defEnvDIESE_sh .
  echo "||| "
  echo "||| Add 'bash ~/.defEnvDIESE_sh' to the appropriate login shell file"
  echo "    (e.g. .bashrc or .profile)"
  echo "||| "
  $EDITOR .*shrc*
fi
echo "||| "
echo "||| Customization of .defEnvDIESE_sh:"
echo "||| - replace <major> and <minor> with 5 and 9 respectively"
echo "||| - uncomment lines according to the locally running operating system"
echo "|||   (pay special attention to the library path section under Cygwin)"
echo "||| - replace <pathDIESE> and <pathKBS> with appropriate pathnames"
echo "||| - SO: search for the local library with the linux 'find' command"
echo "||| - BUFFEREDWRITER_SIZE: remember: may demand a higher value if"
echo "|||           the Makefile generation fails when developping a KSB"
echo "||| "
echo "||| Ready for editing ? (Y/N)"
read goonEditing
if [ $goonEditing != Y ] && [ $goonEditing != y  ]
then
exit
fi
$EDITOR .defEnvDIESE_sh
rm -f .defEnvDIESE_sh~
. ~/.defEnvDIESE_sh

if [ -e "$DIESE_TOOLS" ]
then
      onContinue=0
else
      echo "... $DIESE_TOOLS does not exist. Try another path ..."
fi

if [ -e "$DIESE_KBS" ]
then
      onContinue=0
else
      echo "...$DIESE_KBS does not exist. Try another path ..."
fi
# ... }
done

echo "||| "
echo "||| Installation of DIESE.$MAJOR.$MINOR is in progress ..."
echo "||| DIESE_TOOLS : $DIESE_TOOLS"
echo "||| DIESE_KBS   : $DIESE_KBS"
echo "||| "

# create libDIESE if first time
#--------------------------
cd "$DIESE_TOOLS"
mkdir -p libDIESE

# move lib DIESE to its final destination
#----------------------------------------
cd libDIESE
SRC_LIB="$SOURCE"/bdiese/lib
/bin/cp "$SRC_LIB"/* .
SRC_LIB="$SOURCE"/cdiese/lib
/bin/cp "$SRC_LIB"/* .
SRC_LIB="$SOURCE"/odiese/lib
/bin/cp "$SRC_LIB"/* .
cd ~

#create the final release destination
#------------------------------------
cd "$DIESE_TOOLS"
# --- create DIESE.M.m if new one
DEST=DIESE.$MAJOR.$MINOR
mkdir -p "$DEST"
cd "$DEST"
/bin/cp "$SOURCE"/READ_ME .

# move include DIESE to its final destination
#--------------------------------------------
mkdir -p bdiese
cd bdiese
rm -f -r include
SRC_INC="$SOURCE"/bdiese/include
/bin/cp -r "$SRC_INC" .
cd ..
mkdir -p cdiese
cd cdiese
rm -f -r include
SRC_INC="$SOURCE"/cdiese/include
/bin/cp -r "$SRC_INC" .
cd ..
mkdir -p odiese
cd odiese
rm -f -r include
SRC_INC="$SOURCE"/odiese/include
/bin/cp -r "$SRC_INC" .
cd ..

# move solfege midiese to their final destination
#------------------------------------------------
echo "||| Solfege and MI_Diese have been compiled to be handled"
echo "||| by a java 1.5 VM. Older JRE versions must be upgraded."
mkdir -p solfege
cd solfege
SRC_JAR="$SOURCE"/solfege/solfege.jar
/bin/cp "$SRC_JAR" .
# ci-dessous pour 4.1 4.2
#SRC_JAR="$SOURCE"/solfege/entityConstructorTranslator.jar
#/bin/cp "$SRC_JAR" .
cd ..
mkdir -p midiese
cd midiese
SRC_JAR="$SOURCE"/midiese/midiese.jar
/bin/cp "$SRC_JAR" .

# move doc DIESE to its final destination
#----------------------------------------
cd "$DIESE_TOOLS"
mkdir -p doc
cd doc
mkdir -p bdiese
cd bdiese
SRC_DOC="$SOURCE"/bdiese/doc/html
rm -f -r html
/bin/cp -r "$SRC_DOC" .
cd ..
mkdir -p cdiese
cd cdiese
SRC_DOC="$SOURCE"/cdiese/doc/html
rm -f -r html
/bin/cp -r "$SRC_DOC" .
cd ..
mkdir -p odiese
cd odiese
SRC_DOC="$SOURCE"/odiese/doc/html
rm -f -r html
/bin/cp -r "$SRC_DOC" .
cd ..
mkdir -p solfege
cd solfege
SRC_DOC="$SOURCE"/solfege/docUser
rm -f -r docUser
/bin/cp -r "$SRC_DOC" .
cd ..
mkdir -p midiese
cd midiese
SRC_DOC="$SOURCE"/midiese/docUser
rm -f -r docUser
/bin/cp -r "$SRC_DOC" .

# create KBS/ SIM/ libKBS/ if needed
#-------------------------------
cd "$DIESE_KBS"
mkdir -p KBS
cd KBS
mkdir -p bdiese
mkdir -p cdiese
mkdir -p odiese
cd ..
mkdir -p SIM
cd SIM
mkdir -p bdiese
mkdir -p cdiese
mkdir -p odiese
cd ..
mkdir -p libKBS

# move scripts APPLI to their final destination
#----------------------------------------
cd "$DIESE_KBS"
mkdir -p scripts
cd scripts
rm -f install_APPLI*
rm -f READ_ME_APPLI*
/bin/cp "$SOURCE"/scripts_APPLI/install_APPLI* .
/bin/cp "$SOURCE"/scripts_APPLI/READ_ME_APPLI* .
mkdir -p makeRelease
cd makeRelease
rm -f *
/bin/cp "$SOURCE"/scripts_APPLI/makeRelease_APPLI* .
cd ../..
mkdir -p releases
mkdir -p releases/src
mkdir -p releases/src/LinuxCygwin
mkdir -p releases/lib
mkdir -p releases/lib/Linux
mkdir -p releases/lib/Cygwin

# back to final destination
#--------------------------
cd "$DIESE_TOOLS/$DEST"

# removing release archive
#-------------------------
echo "||| "
echo "||| Do you want to delete the source release directory content (Y/N)?"
echo "||| ($SOURCE)"
echo "||| (excepted the currently running install_DIESE_lib.sh command"
echo "||| and the READ_ME file to be deleted later)"
echo "||| "
read delsource
if [ $delsource = Y ] || [ $delsource = y ] 
then
  rm -f "$SOURCE"/.defEnvDIESE*
  rm -f "$SOURCE"/install_DIESE_lib.csh
  rm -f -r "$SOURCE"/bdiese
  rm -f -r "$SOURCE"/cdiese
  rm -f -r "$SOURCE"/odiese
  rm -f -r "$SOURCE"/midiese
  rm -f -r "$SOURCE"/solfege
fi
echo "||| "
echo "||| Do you want to delete the source release archive (Y/N)?"
echo "||| ($SOURCE.tar)"
echo "||| "
read delsource
if [ $delsource = Y ] || [ $delsource = y ] 
then
 rm -f "$SOURCE".tar
fi
echo "||| "
echo "||| Installation of $DIESE_TOOLS/$DEST done."
echo "||| Type any character then 'Enter' to exit."
echo "||| "
read tmp
