//============================================================================
// FILE: O_CoreException.h
//
// TYPE: C-LIBRARY
// DESCRIPTION: Pseudo-exception catching functions
// 
// AUTHORS: Rellier (INRA/UBIA, Toulouse)
// REFERENCE : 
//=X===============X=============================X============================

#ifndef _O_COREEXCEPTION_HH
#define _O_COREEXCEPTION_HH

// SYSTEM INCLUDES
//


// PROJECT INCLUDES
//
#include "O_CoreTypes.h"


/**@name La gestion des situations d'erreur
Un certain nombre de méthodes (au sens de C++), dans diverses classes, savent reconnaître des situations particulières (en général indésirables) portant sur leurs paramètres ou sur l'état des entités qu'elles manipulent. Dans une telle situation, la méthode en question {\em peut lever une exception}\index{exception}.\\

Le manuel de \Db justifie que le mécanisme standard de reconnaissance et de traitement d'exception de C++ a été remplacé par des appels à des fonctions globales qui réalisent le même effet, mais qui provoquent systématiquement d'arrêt du programme. Ces fonctions sont documentées ci-après pour ce qui concerne les exceptions soulevées par le code de \ODb et seulement lui.
 */
//@{

/**@name Messages d'erreur li\'es aux InterfaceDataItem's */
//@{
         /// Exception levée lorsqu'on tente d'accéder à la valeur attachée à une instance de InterfaceDataItem avec une méthode qui ne correspond pas au type réel de cette valeur.
void BadInterfaceDataItemValueType(Entity* describedEntity, int valueType, const char*  candidateAccess);

         /// Exception levée lorsqu'on a demandé au simulateur la fourniture d'une valeur de descripteur pour une date postérieure à la fin de l'épisode de simulation.
void BadInterfaceDataItemOutClockValue(const char* entityName, int descId, int outClockValue, int endClock);

//@}  // fin messages


//@} // fin Exceptions odiese



#endif  // _O_COREEXCEPTION_HH
