//============================================================================
// FILE: CoreArgument.h
//
// TYPE: C++CLASS 
// DESCRIPTION: entête des arguments des méthodes "moteur"
// 
// AUTHORS: Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _COREARGUMENT_HH
#define _COREARGUMENT_HH

// SYSTEM INCLUDES
//
//#include <>                                    //

// PROJECT INCLUDES
//
#include "Argument.h"

/**@name CDPFORTEXArguments pr\'ed\'efinis des m\'ethodes
\begin{itemize} \item Directive d'inclusion : 
[P]
\#include CoreArgument.h
[/P]
\end{itemize}
C'est normalement l'utilisateur qui crée des descendants de la classe de base, en fonction de son domaine d'application. Cependant, les méthodes prédéfinies dans \Db requièrent des arguments particuliers, qui nécessitent d'être eux aussi prédéfinis dans \D. Les classes présentées dans cette section sont des sous-classes de [C]Argument[/C] utilisées par les méthodes prédéfinies.\\

{\large {\tt Remarque :}} ces classes d'argument de méthodes peuvent aussi être exploitées dans le code d'une application, pour un motif qui lui est propre.
*/ 
//@{

/**@name Arguments de méthodes d'événements
 */
//@{

/**@name IntLowBoundArgument
Un tel argument est donné à la méthode de tirage aléatoire selon la loi uniforme entre deux bornes entières. Il représente la borne inférieure.\index{IntLowBoundArgument}\\ 
*/
class IntLowBoundArgument : public IntArgument
{
public:
          /// Constructeur. \\Affecte le symbole de classe INT\_LOW\_BOUND\_ARGUMENT et le nom de classe "intLowBoundArgument".
   IntLowBoundArgument();           
 }; // _end class_

/**@name IntUpperBoundArgument
Un tel argument est donné à la méthode de tirage aléatoire selon la loi uniforme entre deux bornes entières. Il représente la borne supérieure.\index{IntUpperBoundArgument}\\ 
*/
class IntUpperBoundArgument : public IntArgument
{
public:
          /// Constructeur. \\Affecte le symbole de classe INT\_UPPER\_BOUND\_ARGUMENT et le nom de classe "intUpperBoundArgument".
   IntUpperBoundArgument();           
 }; // _end class_

/**@name PreviousEventArgument
Un tel argument est donné aux la méthode de mise à jour (complète ou seulement sur la date d'occurrence) d'un événement auto-généré. Il représente l'événement à partir duquel l'autogénération s'effcetue.\index{PreviousEventArgument}\\ 
*/
class PreviousEventArgument : public EventArgument
{
public:
          /// Constructeur. \\Affecte le symbole de classe PREVIOUS\_EVENT\_ARGUMENT et le nom de classe "previousEventArgument".
   PreviousEventArgument();           
 }; // _end class_


//@}

/**@name Arguments de méthodes de démons
 */
//@{

/**@name MonitoredDescriptorArgument
Un tel argument est donné aux méthodes de démons, celles dont les corps sont invoqués en réaction à un événement d'accès ou d'affectation de valeur à un descripteur. C'est justement le pointeur sur ce descripteur qui est la valeur de l'argument.\index{MonitoredDescriptorArgument}\\ 
*/
class MonitoredDescriptorArgument : public DescriptorArgument
{
public:
          /// Constructeur. \\Affecte le symbole de classe MONITOREDDESCRIPTOR\_ARGUMENT et le nom de classe "monitoredDescriptorArgument".
   MonitoredDescriptorArgument();           
 }; // _end class_

/**@name CurrentIntValueArgument
Un tel argument est donné aux méthodes de démons dont le corps doit connaître la valeur courante du descripteur à valeur entière. \index{CurrentIntValueArgument}\\ 
*/
class CurrentIntValueArgument : public IntArgument
{
public:
          /// Constructeur. \\Affecte le symbole de classe CURRENTINTVALUE\_ARGUMENT et le nom de classe "currentIntValueArgument".
   CurrentIntValueArgument();           
 }; // _end class_

/**@name CurrentFloatValueArgument
Un tel argument est donné aux méthodes de démons dont le corps doit connaître la valeur courante du descripteur à valeur flottante. \index{CurrentFloatValueArgument}\\ 
*/
class CurrentFloatValueArgument : public FloatArgument
{
public:
          /// Constructeur. \\Affecte le symbole de classe CURRENTFLOATVALUE\_ARGUMENT et le nom de classe "currentFloatValueArgument".
   CurrentFloatValueArgument();           
 }; // _end class_

/**@name CurrentDoubleValueArgument
Un tel argument est donné aux méthodes de démons dont le corps doit connaître la valeur courante du descripteur à valeur flottante double précision. \index{CurrentDoubleValueArgument}\\ 
*/
class CurrentDoubleValueArgument : public DoubleArgument
{
public:
          /// Constructeur. \\Affecte le symbole de classe CURRENTDOUBLEVALUE\_ARGUMENT et le nom de classe "currentDoubleValueArgument".
   CurrentDoubleValueArgument();           
 }; // _end class_

/**@name CurrentStringValueArgument
Un tel argument est donné aux méthodes de démons dont le corps doit connaître la valeur courante du descripteur à valeur 'chaîne de caractères'. \index{CurrentStringValueArgument}\\ 
*/
class CurrentStringValueArgument : public StringArgument
{
public:
          /// Constructeur. \\Affecte le symbole de classe CURRENTSTRINGVALUE\_ARGUMENT et le nom de classe "currentStringValueArgument".
   CurrentStringValueArgument();           
 }; // _end class_

/**@name CurrentEntityValueArgument
Un tel argument est donné aux méthodes de démons dont le corps doit connaître la valeur courante du descripteur à valeur pointeur sur une entité. \index{CurrentEntityValueArgument}\\ 
*/
class CurrentEntityValueArgument : public EntityArgument
{
public:
          /// Constructeur. \\Affecte le symbole de classe CURRENTENTITYVALUE\_ARGUMENT et le nom de classe "currentEntityValueArgument".
   CurrentEntityValueArgument();           
 }; // _end class_

/**@name CurrentEntityTabValueArgument
Un tel argument est donné aux méthodes de démons dont le corps doit connaître la valeur courante du descripteur à valeur pointeur sur un tableau d'entités. \index{CurrentEntityTabValueArgument}\\ 
*/
class CurrentEntityTabValueArgument : public EntityTabArgument
{
public:
          /// Constructeur. \\Affecte le symbole de classe CURRENTENTITYTABVALUE\_ARGUMENT et le nom de classe "currentEntityTabValueArgument".
   CurrentEntityTabValueArgument();           
 }; // _end class_

/**@name CurrentDescValueSpecTabValueArgument
Un tel argument est donné aux méthodes de démons dont le corps doit connaître la valeur courante du descripteur à valeur pointeur sur un tableau de domaines de valeurs. \index{CurrentDescValueSpecTabValueArgument}\\ 
*/
class CurrentDescValueSpecTabValueArgument : public DescValueSpecTabArgument
{
public:
          /// Constructeur. \\Affecte le symbole de classe CURRENTDESCVALUESPECTABVALUE\_ARGUMENT et le nom de classe "currentDescValueSpecTabValueArgument".
   CurrentDescValueSpecTabValueArgument();           
 }; // _end class_

/**@name CandidateIntValueArgument
Un tel argument est donné aux méthodes de démons dont le corps doit connaître la valeur candidate pour le descripteur à valeur entière.\index{CandidateIntValueArgument}\\
*/
class CandidateIntValueArgument : public IntArgument
{   
public:
          /// Constructeur. \\Affecte le symbole de classe CANDIDATEINTVALUE\_ARGUMENT et le nom de classe "candidateIntValueArgument".
   CandidateIntValueArgument();           
 }; // _end class_

/**@name  CandidateFloatValueArgument
Un tel argument est donné aux méthodes de démons dont le corps doit connaître la valeur candidate pour le descripteur à valeur flottante. \index{CandidateFloatValueArgument}\\
*/
class CandidateFloatValueArgument : public FloatArgument
{
public:
          /// Constructeur. \\Affecte le symbole de classe CANDIDATEFLOATVALUE\_ARGUMENT et le nom de classe "candidateFloatValueArgument".
   CandidateFloatValueArgument();           
 }; // _end class_

/**@name CandidateDoubleValueArgument
Un tel argument est donné aux méthodes de démons dont le corps doit connaître la valeur candidate pour le descripteur à valeur flottante double précision. \index{CandidateDoubleValueArgument}\\ 
*/
class CandidateDoubleValueArgument : public DoubleArgument
{
public:
          /// Constructeur. \\Affecte le symbole de classe CANDIDATEDOUBLEVALUE\_ARGUMENT et le nom de classe "candidateDoubleValueArgument".
   CandidateDoubleValueArgument();           
 }; // _end class_

/**@name  CandidateStringValueArgument
Un tel argument est donné aux méthodes de démons dont le corps doit connaître la valeur candidate pour le descripteur à valeur 'chaîne de caractères'. \index{CandidateStringValueArgument}\\ 
*/
class CandidateStringValueArgument : public StringArgument
{
public:
          /// Constructeur. \\Affecte le symbole de classe CANDIDATESTRINGVALUE\_ARGUMENT et le nom de classe "candidateStringValueArgument".
   CandidateStringValueArgument();           
 }; // _end class_

/**@name CandidateEntityValueArgument
Un tel argument est donné aux méthodes de démons dont le corps doit connaître la valeur candidate pour le descripteur à valeur pointeur sur une entité. \index{CandidateEntityValueArgument}
*/
class CandidateEntityValueArgument : public EntityArgument        
{
public:
          /// Constructeur. \\Affecte le symbole de classe CANDIDATEENTITYVALUE\_ARGUMENT et le nom de classe "candidateEntityValueArgument".
   CandidateEntityValueArgument();  
 }; // _end class_ 

/**@name CandidateEntityTabValueArgument
Un tel argument est donné aux méthodes de démons dont le corps doit connaître la valeur candidate pour le descripteur à valeur pointeur sur un tableau d'entités. \index{CandidateEntityTabValueArgument}\\ 
*/
class CandidateEntityTabValueArgument : public EntityTabArgument
{
public:
          /// Constructeur. \\Affecte le symbole de classe CANDIDATEENTITYTABVALUE\_ARGUMENT et le nom de classe "candidateEntityTabValueArgument".
   CandidateEntityTabValueArgument();           
 }; // _end class_

/**@name CandidateDescValueSpecValueArgument
Un tel argument est donné aux méthodes de démons dont le corps doit connaître la valeur candidate pour le descripteur à valeur pointeur sur un domaine de valeur. \index{CandidateDescValueSpecValueArgument}\\ 
*/
class CandidateDescValueSpecValueArgument : public DescValueSpecArgument
{
public:
          /// Constructeur. \\Affecte le symbole de classe CANDIDATEDESCVALUESPECVALUE\_ARGUMENT et le nom de classe "candidateDescValueSpecValueArgument".
   CandidateDescValueSpecValueArgument();           
 }; // _end class_

/**@name CandidateDescValueSpecTabValueArgument
Un tel argument est donné aux méthodes de démons dont le corps doit connaître la valeur candidate pour le descripteur à valeur pointeur sur un tableau de domaines de valeurs. \index{CandidateDescValueSpecTabValueArgument}\\
*/
class CandidateDescValueSpecTabValueArgument : public DescValueSpecTabArgument
{
public:
          /// Constructeur. \\Affecte le symbole de classe CANDIDATEDESCVALUESPECTABVALUE\_ARGUMENT et le nom de classe "candidateDescValueSpecTabValueArgument".
   CandidateDescValueSpecTabValueArgument();           
 }; // _end class_

/**@name CurrentEntityContainerArgument
Un tel argument est donné aux méthodes de démons dont le corps doit connaître l'entité dont on touche la liste des éléments. \index{CurrentEntityContainerArgument}\\ 
*/
class CurrentEntityContainerArgument : public EntityArgument
{
public:
          /// Constructeur. \\Affecte le symbole de classe CURRENTENTITYCONTAINER\_ARGUMENT et le nom de classe "currentEntityContainerArgument".
   CurrentEntityContainerArgument();           
 }; // _end class_

/**@name CandidateEntityElementArgument
Un tel argument est donné aux méthodes de démons dont le corps doit connaître l'entité qu'on tente d'ajouter ou de retirer d'une liste d'éléments d'une autre. \index{CandidateEntityElementArgument}\\ 
*/
class CandidateEntityElementArgument : public EntityArgument
{
public:
          /// Constructeur. \\Affecte le symbole de classe CANDIDATEENTITYELEMENT\_ARGUMENT et le nom de classe "candidateEntityElementArgument".
   CandidateEntityElementArgument();           
 }; // _end class_


//@}

/**@name Arguments de méthodes de spécifications d'ensembles d'entités
 */
//@{

/**@name ReferenceEntityArgument
Un tel argument est donné au prédicat de sélection d'une spécification d'ensemble d'entités, ou à une de ses fonction de construction, ou encore à sa fonction qui renvoie le cardinal de la liste d'entites à construire.\index{ReferenceEntityArgument}\\ 
*/
class ReferenceEntityArgument : public EntityArgument
{
public:
          /// Constructeur. \\Affecte le symbole de classe REFERENCEENTITY\_\-ARGUMENT et le nom de classe "referenceEntityArgument".
   ReferenceEntityArgument();           
 }; // _end class_

/**@name CandidateEntityArgument
Un tel argument est donné au prédicat de sélection d'une spécification d'ensemble d'entités.\\
Un tel argument est aussi donné au prédicat qui teste si une entité satisfait la définition d'une spécification de domaines de valeurs. \index{CandidateEntityArgument}\\ 
*/
class CandidateEntityArgument : public EntityArgument
{
public:
          /// Constructeur. \\Affecte le symbole de classe CANDIDATEENTITY\_\-ARGUMENT et le nom de classe "candidateEntityArgument".
   CandidateEntityArgument();           
 }; // _end class_

/**@name ClassIdArgument
Un tel argument est donné aux fonctions de construction d'une spécification d'ensemble d'entités.\index{ClassIdArgument}\\ 
*/
class ClassIdArgument : public IntArgument
{
public:
          /// Constructeur. \\Affecte le symbole de classe CLASSID\_ARGUMENT et le nom de classe "classIdArgument".
   ClassIdArgument();           
 };   // _end class_

/**@name ListSizeArgument
Un tel argument est donné à la fonction de construction de la liste des entités répondant à une  spécification d'ensemble d'entités.\index{ListSizeArgument}\\ 
*/
class ListSizeArgument : public IntArgument
{
public:
          /// Constructeur. \\Affecte le symbole de classe LISTSIZE\_ARGUMENT et le nom de classe "listSizeArgument".
   ListSizeArgument();           
 }; // _end class_


//@}

/**@name Arguments prédéfinis sans destination dans \D

Ces arguments ont une sémantique générale, qui rend envisageable leur utilité dans un domaine d'application.
 */
//@{

/**@name DescriptorIdArgument
Cet argument reçoit pour valeur un identificateur de classe de descripteur. \Db n'en fait pas usage dans son propre code.\index{DescriptorIdArgument}\\ 
*/
class DescriptorIdArgument : public IntArgument
{
public:
          /// Constructeur. \\Affecte le symbole de classe DESCRIPTORID\_ARGUMENT et le nom de classe "classIdArgument".
   DescriptorIdArgument();           
 }; // _end class_

/**@name ClockValueArgument
Cet argument reçoit pour valeur une valeur de l'horloge de simulation (valeur actuelle ou non). \Db n'en fait pas usage dans son propre code.\index{DateArgument}\\ 
*/

class ClockValueArgument : public IntArgument      // unused
{
public:
          /// Constructeur. \\Affecte le symbole de classe CLOCK\_VALUE\_ARGUMENT et le nom de classe "clockValueArgument".
   ClockValueArgument();           
 }; // _end class_


//@}

//@}  // fin arguments prédéfinis

//@}  // fin arguments

#endif  // COREARGUMENT_HH
